#! /bin/sh
# -*- mode:sh -*-
# $Id: denycomm.sh,v 1.6 2003/12/28 23:42:22 user Exp $
# Copyright (c) 2003 auto92089@hushmail.com

# This script denies communication to an IP address.

# set base filename for prepending to errors
progname=$(basename "$0")

# default config file
config="/etc/$progname.conf"

# usage info
usage="\
$progname [-iprt] address_specification
Blocks an IP address specification.  Specification format depends on command:
-i use ipf (assumes appending a rule works)
-p use pf (assumes appending a rule works)
-r use route (assumes this form: route add HOST 127.0.0.1 1)
-t use iptables (assumes chain named DENYCOMM called by INPUT and/or FORWARD)
If none of these options are specified, it processes $config.
"

ipf=false
pf=false
route=false
iptables=false

while getopts "iptr" c
do
    case "$c" in
    i)
	ipf=true
	;;
    p)
	pf=true
	;;
    r)
	route=true
	;;
    t)
	iptables=true
	;;
    *)
	echo "invalid option"
	;;
    esac
done

shift $(($OPTIND - 1))

if test $# -lt 1
then
    echo "$usage"
    exit 1
fi

cmd_ipf () {
    echo "block in quick from $1 to any" | ipf -f /dev/stdin
}

cmd_pf () {
    echo "block in quick from $1 to any" | pf -f /dev/stdin
}

cmd_route () {
    route add "$1" 127.0.0.1 1
}

cmd_iptables () {
    iptables -A DENYCOMM -j DROP -s "$1"
}

cmd_custom () {
    eval $(/usr/bin/printf "$@")
}

process_config_line () {
    # Ignore comments.
    expr "$*" : ' *#' > /dev/null && return
    # Ignore blank lines.
    expr x"$*" : x' *$' > /dev/null && return
    # Look at the first field.
    command="$1"
    shift
    case "$command" in
	pf|ipf|route|iptables|custom)
	    # NOTE: Any arguments in the config file are passed
	    #       along before addr_spec
	    "cmd_$command" "$@"
	    ;;
	*)
	    echo "Unrecognized config line: $line" >&2
	    ;;
    esac
}

process_config () {
    if exec < $config
    then
	while read -r line
	do
	    eval process_config_line "$line" "$@"
	done
    else
	echo "Error: no commands specified and no readable $config." >&2
	exit 1
    fi
}

if $ipf || $pf || $route || $iptables
then
    $ipf && cmd_ipf "$1"
    $pf && cmd_pf "$1"
    $route && cmd_route "$1"
    $iptables && cmd_iptables "$1"
else
    process_config "$1"
fi

exit 0
